@@ -20,11 +20,26 @@ class MergeKeysMismatchError(Exception):
         Exception.__init__(self, "multisig parameters do not match")
 
 
+class MergeAuthAddrMismatchError(Exception):
+    def __init__(self):
+        Exception.__init__(self, "multisig transaction auth addresses do not match")
+
+
 class DuplicateSigMismatchError(Exception):
     def __init__(self):
         Exception.__init__(self, "mismatched duplicate signatures in multisig")
 
 
+class LogicSigOverspecifiedSignature(Exception):
+    def __init__(self):
+        Exception.__init__(self, "LogicSig has too many signatures. At most one of sig or msig may be present")
+
+
+class LogicSigSigningKeyMissing(Exception):
+    def __init__(self):
+        Exception.__init__(self, "LogicSigAccount is missing signing key")
+
+
 class WrongAmountType(Exception):
     def __init(self):
         Exception.__init__(self, "amount (amt) must be a non-negative integer")
@@ -135,6 +150,11 @@ class ZeroAddressError(Exception):
             "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAY5HFKQ")
 
 
+class KeyregOnlineTxnInitError(Exception):
+    def __init__(self, attr):
+        Exception.__init__(self, attr + " should not be None")
+
+
 class TemplateInputError(Exception):
     pass
 
